/**
 * Check out https://googlechrome.github.io/sw-toolbox/docs/master/index.html for
 * more info on how to use sw-toolbox to custom configure your service worker.
 */

'use strict';

const DEBUG = false;
const BASE_URL = 'https://savelist.co/';
const NOTIFICATIONS_SEGMENT = 'notifications/activity';

// Bypass insecure HTTP requests
self.addEventListener('fetch', event => {
  if (event.request.url.indexOf('http://') === 0) {
    event.stopImmediatePropagation();
    DEBUG && console.log(`Bypassing insecure request to "${event.request.url}"`);
  }
});

importScripts('./build/sw-toolbox.js');

self.toolbox.options = {
  debug: DEBUG,
  networkTimeoutSeconds: 30,
  cache: {
    name: 'savelist-cache',
    maxAgeSeconds: 60 * 60 * 24 // 1 day
  }
};

// Pre-cache our key assets
self.toolbox.precache(
  [
    './build/main.js',
    './build/vendor.js',
    './build/main.css',
    './build/polyfills.js',
    'index.html',
    'manifest.json'
  ]
);

// Dynamically cache any other local assets
self.toolbox.router.any('/assets/*', self.toolbox.networkFirst);
self.toolbox.router.any('/build/*', self.toolbox.networkFirst);

// Savelist API
const apiOpts = { origin: 'https://api.savelist.co' };
self.toolbox.router.get('/*', self.toolbox.networkFirst, apiOpts);

// For any other requests go to the network
self.toolbox.router.default = self.toolbox.networkOnly;

// Google Analytics
// https://developers.google.com/web/ilt/pwa/integrating-analytics

function logEvent(name) {
  return self.registration.pushManager.getSubscription()
  .then(subscription => {
    if (!subscription) {
      throw new Error('No subscription currently available.');
    }

    const payloadData = {
      v: 1,
      cid: subscription.endpoint,
      tid: 'UA-55668864-5',
      ds: 'co.savelist.web',
      t: 'event',
      ec: 'service-worker',
      ea: 'click',
      el: name
    };

    const payloadString = Object.keys(payloadData)
    .filter(key => payloadData[key])
    .map(key => key + '=' + encodeURIComponent(payloadData[key]))
    .join('&');

    return fetch('https://www.google-analytics.com/collect', {
      method: 'POST',
      body: payloadString,
      mode: 'cors'
    });
  })
  .catch(err => DEBUG && console.error(err));
}

// Web Push Notifications

self.addEventListener('push', event => {
  // Convert notification data to JSON
  const notificationData = event.data.json();
  const notificationOptions = {
    body: notificationData.message,
    icon: notificationData.image,
    data: notificationData,
    tag: notificationData.notId,
    renotify: Boolean(notificationData.notId)
  };

  self.registration.showNotification(notificationData.title, notificationOptions);
});

self.addEventListener('notificationclick', event => {
  const notification = event.notification;
  notification.close();

  event.waitUntil(logEvent('sl_notification_open'));

  if (notification && notification.data && notification.data.deeplink) {
    event.waitUntil(clients.openWindow(notification.data.deeplink));
  } else {
    event.waitUntil(clients.openWindow(BASE_URL + NOTIFICATIONS_SEGMENT));
  }
});
